//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "InternalForm.h"
#include <IniFiles.hpp>
#include "StringHelp.h"
#include <SysUtils.hpp>
#include <math.h>
#include "MathsHelp.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TIntForm *IntForm;
//---------------------------------------------------------------------------
__fastcall TIntForm::TIntForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TIntForm::FormCreate(TObject *Sender)
{
    //wire up hints so all appear in the status bar
    Application->OnHint = StatusBar1Hint;
    
    //Open Persistant Data & initialise variables
    TIniFile *IniFile = new TIniFile(ExtractFileDir(Application->ExeName) + "\\Data.ini");
    toolD = IniFile->ReadFloat("ThreadMill","toolD",6.35);
    toolD1 = IniFile->ReadFloat("ThreadMill","toolD1",4);
    toolL = IniFile->ReadFloat("ThreadMill","toolL",12.5);
    toolL1 = IniFile->ReadFloat("ThreadMill","toolL1",1.5);
    toolN = IniFile->ReadInteger("ThreadMill","toolN",0);
    showG90point1 = IniFile->ReadBool("ThreadMill","Use G90.1",true);

    F = IniFile->ReadFloat("InternalThread","F",100.0);
    CX = IniFile->ReadFloat("InternalThread","CX",0.0);
    CY = IniFile->ReadFloat("InternalThread","CY",0.0);
    CZ = IniFile->ReadFloat("InternalThread","CZ",0.0);
    SA = IniFile->ReadFloat("InternalThread","SA",0.0);
    N = IniFile->ReadInteger("InternalThread","N",1);
    D = IniFile->ReadFloat("InternalThread","D",12.0);
    G = IniFile->ReadFloat("InternalThread","G",2.0);
    L = IniFile->ReadFloat("InternalThread","L",10.0);
    if (L > toolL) L = toolL;
    P = IniFile->ReadFloat("InternalThread","P",1.0);
    CentreType = (TCentreType) IniFile->ReadInteger("InternalThread","CentreType",0);
    ClimbMill = IniFile->ReadBool("InternalThread","ClimbMill",true);

    GCodeHeader = IniFile->ReadString("GCode","Header","");
    GCodeFooter = IniFile->ReadString("GCode","Footer","M2");

    ThreadFit = TFitNone;
    ThreadDirection = TRightHand;

    delete IniFile;

    UpdateEditBoxes();
    recalconchange = true;
    SetRecalculationStatus(recalconchange);
    Recalculate(this);

}
//---------------------------------------------------------------------------
void __fastcall TIntForm::FormDestroy(TObject *Sender)
{
    //Save Persistant Data
    TIniFile *IniFile = new TIniFile(ExtractFileDir(Application->ExeName) + "\\Data.ini");
    IniFile->WriteFloat("ThreadMill","toolD",toolD);
    IniFile->WriteFloat("ThreadMill","toolD1",toolD1);
    IniFile->WriteFloat("ThreadMill","toolL",toolL);
    IniFile->WriteFloat("ThreadMill","toolL1",toolL1);
    IniFile->WriteInteger("ThreadMill","toolN",toolN);
    IniFile->WriteBool("ThreadMill","Use G90.1",showG90point1);

    IniFile->WriteFloat("InternalThread","F",F);
    IniFile->WriteFloat("InternalThread","CX",CX);
    IniFile->WriteFloat("InternalThread","CY",CY);
    IniFile->WriteFloat("InternalThread","CZ",CZ);
    IniFile->WriteFloat("InternalThread","SA",SA);
    IniFile->WriteInteger("InternalThread","N",N);
    IniFile->WriteFloat("InternalThread","D",D);
    IniFile->WriteFloat("InternalThread","G",G);
    IniFile->WriteFloat("InternalThread","L",L);
    IniFile->WriteFloat("InternalThread","P",P);
    IniFile->WriteInteger("InternalThread","CentreType",(int) CentreType);
    IniFile->WriteBool("InternalThread","ClimbMill",ClimbMill);

    IniFile->WriteString("GCode","Header",GCodeHeader);
    IniFile->WriteString("GCode","Footer",GCodeFooter);

    delete IniFile;

    //stop un-necessaery work being done as each edit box is destroyed.
    SetRecalculationStatus(false);

}
//---------------------------------------------------------------------------
void __fastcall TIntForm::Exit1Click(TObject *Sender)
{
    Close();    
}
//---------------------------------------------------------------------------
void __fastcall TIntForm::File1Click(TObject *Sender)
{
    //Checks to see if there is anything in the memo box to save.
    Save1->Enabled = (bool) Memo1->Lines->Count;
}
//---------------------------------------------------------------------------
void __fastcall TIntForm::Save1Click(TObject *Sender)
{
    if (SaveDialog1->Execute()) {
        Memo1->Lines->SaveToFile(SaveDialog1->FileName);
        }
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::UpdateEditBoxes(void)
{
    //Updates edit boxes to reflect current content of data held
    SetRecalculationStatus(false);
    toolDEdit->Text = AnsiString(toolD);
    toolD1Edit->Text = AnsiString(toolD1);
    toolLEdit->Text = AnsiString(toolL);
    toolL1Edit->Text = AnsiString(toolL1);
    toolNEdit->Text = AnsiString(toolN);

    CXEdit->Text = AnsiString(CX);
    CYEdit->Text = AnsiString(CY);
    CZEdit->Text = AnsiString(CZ);
    FEdit->Text = AnsiString(F);
    SAEdit->Text = AnsiString(SA);
    DEdit->Text = AnsiString(D);
    GEdit->Text = AnsiString(G);
    LEdit->Text = AnsiString(L);
    NEdit->Text = AnsiString(N);
    PEdit->Text = AnsiString(P);

    GCodeHeaderEdit->Text = GCodeHeader;
    GCodeFooterEdit->Text = GCodeFooter;

    SetRecalculationStatus(recalconchange);

}
//---------------------------------------------------------------------------

void __fastcall TIntForm::UpdateVariables(void)
{
    //Updates edit boxes to reflect current content of data held
    toolD = ToDoubleDef(toolDEdit->Text,6.0);
    toolD1 = ToDoubleDef(toolD1Edit->Text,4.0);
    toolL = ToDoubleDef(toolLEdit->Text,12.0);
    toolL1 = ToDoubleDef(toolL1Edit->Text,2.0);
    toolN = toolNEdit->Text.ToIntDef(0);

    CX = ToDoubleDef(CXEdit->Text,0.0);
    CY = ToDoubleDef(CYEdit->Text,0.0);
    CZ = ToDoubleDef(CZEdit->Text,0.0);
    F = ToDoubleDef(FEdit->Text,100.0);
    SA = ToDoubleDef(SAEdit->Text,0.0);
    D = ToDoubleDef(DEdit->Text,12.0);
    G = ToDoubleDef(GEdit->Text,2.0);
    L = ToDoubleDef(LEdit->Text,toolL);
    if (L > toolL) {
        SetRecalculationStatus(false);
        L = toolL;
        LEdit->Text = AnsiString(L);
        SetRecalculationStatus(recalconchange);
        }
    N = NEdit->Text.ToIntDef(1);
    P = ToDoubleDef(PEdit->Text,0.0);
    //check if there is enough clearance for the tool down the tapping hole
    if ((D - toolD - (14.0 * P * 0.866025404)/(8.0 * N))< 0.0) {
        SetRecalculationStatus(false);
        D = toolD + (14.0 * P * 0.866025404)/(8.0 * N);
        DEdit->Text = AnsiString(D);
        SetRecalculationStatus(recalconchange);
        }

    GCodeHeader = GCodeHeaderEdit->Text;
    GCodeFooter = GCodeFooterEdit->Text;

}
//---------------------------------------------------------------------------

void __fastcall TIntForm::Copy1Click(TObject *Sender)
{
    //Copy selected G code in Memo Box to clipboard
    Memo1->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::SelectAll1Click(TObject *Sender)
{
    //Select All G code in Memo Box
    Memo1->SelectAll();

}
//---------------------------------------------------------------------------

void __fastcall TIntForm::Help2Click(TObject *Sender)
{
    //A popup help will appear
    AnsiString text = "Fill in all the requested data.\nStart top Left column, work down\nthen next column etc.\nThen save the g code file.\n\nThe 'Lead/Pitch' is the advance per turn of each helical cut.\n\nThe tool zero in the program is the point of the cutter.\nIf zeroing the tool to the work by resting the tool on top\nof the work, then the cutter Z must be set to TL1.";
    text = text + "\n\nA single point cutter or tap may be used to create the threads.";
    text = text + "\n\nUse Climb Milling with a single point cutter and/or a light machine.\nDO NOT use climb milling on RH threads when using a RH tap\nas the milling cutter.";
    Application->MessageBox(text.c_str(),"Help",IDOK);
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::Edit1Click(TObject *Sender)
{
    //Checks to see if there is anything in the memo box to select or copy.
    SelectAll1->Enabled = (bool) Memo1->Lines->Count;
    Copy1->Enabled = (bool) Memo1->SelLength;
    ShowG9011->Checked = showG90point1;
    ClimbMill1->Checked = ClimbMill;

}
//---------------------------------------------------------------------------

void __fastcall TIntForm::SetRecalculationStatus(bool status)
{
    if (status) {
        toolDEdit->OnChange = Recalculate;
        toolD1Edit->OnChange = Recalculate;
        toolLEdit->OnChange = Recalculate;
        toolL1Edit->OnChange = Recalculate;
        toolNEdit->OnChange = Recalculate;
        CXEdit->OnChange = Recalculate;
        CYEdit->OnChange = Recalculate;
        CZEdit->OnChange = Recalculate;
        SAEdit->OnChange = Recalculate;
        DEdit->OnChange = Recalculate;
        FEdit->OnChange = Recalculate;
        GEdit->OnChange = Recalculate;
        LEdit->OnChange = Recalculate;
        NEdit->OnChange = Recalculate;
        PEdit->OnChange = Recalculate;
        GCodeHeaderEdit->OnChange = Recalculate;
        GCodeFooterEdit->OnChange = Recalculate;
        } else {
        toolDEdit->OnChange = NULL;
        toolD1Edit->OnChange = NULL;
        toolLEdit->OnChange = NULL;
        toolL1Edit->OnChange = NULL;
        toolNEdit->OnChange = NULL;
        CXEdit->OnChange = NULL;
        CYEdit->OnChange = NULL;
        CZEdit->OnChange = NULL;
        SAEdit->OnChange = NULL;
        DEdit->OnChange = NULL;
        FEdit->OnChange = NULL;
        GEdit->OnChange = NULL;
        LEdit->OnChange = NULL;
        NEdit->OnChange = NULL;
        PEdit->OnChange = NULL;
        GCodeHeaderEdit->OnChange = NULL;
        GCodeFooterEdit->OnChange = NULL;
        }
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::Recalculate(TObject *Sender)
{
    //this routine is called up to generate the g code evey time there is
    //a change to any of the parameters.
    
    //define vars
    double x;
    double y;
    double z;
    double oldx;
    double oldy;
    double oldz;
    double radius;
    double anglebetweenstarts;
    //these variables needed for non climb milling
    double theta;
    double pi = 3.14159265358979;
    double startangle;
    double endangle;
    double t1 = CZ + G + toolL1 - 0.0001;
       //int numberofturns;

    //temp variable
    AnsiString temp;

    //get variables & validate data
    UpdateVariables();
    if (L > toolL) {
        L = toolL;
        SetRecalculationStatus(false);
        LEdit->Text = AnsiString(L);
        SetRecalculationStatus(recalconchange);
        }
    if (SA < 0.0) {
        while (SA < 0.0) SA += 360.0;
        SetRecalculationStatus(false);
        SAEdit->Text = AnsiString(SA);
        SetRecalculationStatus(recalconchange);
        }
    if (SA >= 360.0) {
        while (SA >= 360.0) SA -= 360.0;
        SetRecalculationStatus(false);
        SAEdit->Text = AnsiString(SA);
        SetRecalculationStatus(recalconchange);
        }

    Memo1->Clear();

    if (P != 0.0 ) {
        //Recalculates the g code
        if (GCodeHeader.Length()) {
            Memo1->Lines->Add("(Header)");
            Memo1->Lines->Add(GCodeHeader);
            }

        Memo1->Lines->Add("(Internal Thread, " + AnsiString(N) + " Start)");
        Memo1->Lines->Add("(Tool Dia " + toolDEdit->Text + "mm)");
        Memo1->Lines->Add("(M"+ DEdit->Text + "x" + PEdit->Text + ", " + LEdit->Text + " Long)");
        Memo1->Lines->Add("(Centre X"+ CXEdit->Text + ", Y" + CYEdit->Text + ", Z" + CZEdit->Text + ", Start Angle " + SAEdit->Text + "deg.)");
        temp = "(Thread Fit Specified: ";
        switch (ThreadFit) {
            case TFitFree:  {
                            temp += "Free Fit - 7H";
                            }
                            break;
            case TFitMedium: {
                            temp += "Medium Fit - 6H";
                            }
                            break;
            case TFitClose: {
                            temp += "Close Fit - 5H";
                            }
                            break;
            default:        {
                            temp += "Machine to Size - default";
                            }
                            break;
            }
        temp += ")";
        Memo1->Lines->Add(temp);
        //state tapping hole dia and tolerance on it
        //From Machinery's Handbook, 22nd edition, pages 1341 to 1345. T6 is the H6 tolerance on minor diameter
        double mindia, T6;
        temp = "(Tapping hole diameter for full engagement: ";
        mindia = D - 2.0 * 0.54127 *P;
        if (P >= 0.9) {
            T6 = 0.433 * P - 0.190 * pow(P,1.22);
            } else {
            T6 = 0.23 * pow(P,0.7);
            }
        switch (ThreadFit) {
            case TFitFree:  {
                            temp += DecimalPlaces(2,mindia) + "mm - " + DecimalPlaces(2,mindia + 1.25 * T6);
                            }
                            break;
            case TFitMedium: {
                            temp += DecimalPlaces(2,mindia) + "mm - " + DecimalPlaces(2,mindia + T6);
                            }
                            break;
            case TFitClose: {
                            temp += DecimalPlaces(2,mindia) + "mm - " + DecimalPlaces(2,mindia + 0.8 * T6);
                            }
                            break;
            default:        {
                            temp += DecimalPlaces(2,mindia);
                            }
                            break;
            }
        temp += "mm)";
        Memo1->Lines->Add(temp);


        //specify XY Plane, Metric & absolute positions for positions and arc definitions
        Memo1->Lines->Add("(XY Plane, Metric units, Abs co-ords, Abs arc centre pos'ns if possible)");
        //if older machines don't support G90.1 or G91.1
        temp = "G17 G21 G90";
        if (showG90point1) temp = temp + " G90.1";
        Memo1->Lines->Add(temp);
        //Memo1->Lines->Add("G17 G21 G90 G90.1");

        //Rapid up to gap height
        z = CZ + G + toolL1;
        Memo1->Lines->Add("(Rapid to Clearance Height)");
        Memo1->Lines->Add("G0 Z" + AnsiString(z));

        //according to metric thread definition, the distance from the surface
        //of an external thread is 7/8 H, where H is 0.866025404 the pitch.
        //As we have N starts to consider the number of threads also has to cut
        //down the depth of cut.
        radius = D/2.0 - toolD/2.0 + (1.0 * P * 0.866025404)/(8.0 * N);
        Memo1->Lines->Add("(Outer Tooltip traces arc dia " + DecimalPlaces(5,2.0*(radius + toolD/2.0)) + "mm to create Metric threadform)");

        //now set the feed rate
        Memo1->Lines->Add("(Set Feed Rate. Surface feed rate = "+ DecimalPlaces(2,F) + "mm/min)");
        Memo1->Lines->Add("(The tool is at a smaller radius, so moves more slowly)");
        Memo1->Lines->Add("F" + DecimalPlaces(2,F * radius * 2/D));
        if (!ClimbMill) {
        ///*

            //Rapid across to start point. Note due to height toolL1 & G
            //the actual cutting point will be above the gap plane, so the point to
            //which the tool goes is not the same as if it starts at angle SA
            startangle = SA * pi/180.0 + (toolL1 + G)*2.0*pi/P;
            //end angle is startangle - L/P * 2*pi
            endangle = SA * pi/180.0 - (L * 2.0 * pi)/P;
            theta = endangle;

            while (theta < startangle ) {
                theta += 2 * pi;
                }
            if (theta > 2*pi) {
                theta -= 2* pi;
                }
            endangle = theta;

            anglebetweenstarts = 2.0 * pi / N;

            for (int i = 0; i < N; i++) {
                theta = startangle + double(i) * anglebetweenstarts;
                x = CX + radius * cos(theta);
                y = CY + radius * sin(theta);
                z = CZ + G + toolL1;

                // rapid to the beginning of this start of the thread
                //this will always be absolute
                Memo1->Lines->Add("(Rapid to Thread " + AnsiString(i + 1) + " Start at clearance height)");
                Memo1->Lines->Add("G0 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y));
                oldx = x;
                oldy = y;
                oldz = z;

                //feed downwards
                theta = endangle + double(i) * anglebetweenstarts;
                x = CX + radius * cos(theta);
                y = CY + radius * sin(theta);
                //namdation, Mach3 seems to allow a maximum of a single circle.
                z += (theta - (startangle + double(i) * anglebetweenstarts)) * P/(2 * pi);
                if (z >= t1) {
                    z -= P;
                    }
                Memo1->Lines->Add("(NOTE Bottom of tool feeds down to Z" + DecimalPlaces(5,CZ - L - toolL1) + " It MUST be clear around thread)");
                Memo1->Lines->Add("(Initial partial Arc)");
                switch (CentreType) {
                    case TAbsolute: {
                                    Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y) + " Z" + DecimalPlaces(5,z) + " I" + DecimalPlaces(5,CX) + " J" + DecimalPlaces(5,CY));
                                    }
                                    break;
                    case TCP1:      {
                                    Memo1->Lines->Add("G91");
                                    if (showG90point1) Memo1->Lines->Add("G91.1");
                                    Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,CX - oldx) + " J" + DecimalPlaces(5,CY - oldy));
                                    }
                                    break;
                    case TP1C:      {
                                    Memo1->Lines->Add("G91");
                                    if (showG90point1) Memo1->Lines->Add("G91.1");
                                    Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,oldx - CX) + " J" + DecimalPlaces(5,oldy - CY));
                                    }
                                    break;
                    case TCP2:      {
                                    Memo1->Lines->Add("G91");
                                    if (showG90point1) Memo1->Lines->Add("G91.1");
                                    Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,CX - x) + " J" + DecimalPlaces(5,CY - y));
                                    }
                                    break;
                    case TP2C:      {
                                    Memo1->Lines->Add("G91");
                                    if (showG90point1) Memo1->Lines->Add("G91.1");
                                    Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,x - CX) + " J" + DecimalPlaces(5,y - CY));
                                    }
                                    break;
                    default:        {
                                    Memo1->Lines->Add("(Unknown Incremental Arc Definition Encountered. Please contact Technical Support at www.chestnutpens.co.uk)");
                                    }
                                    break;
                    } //end switch statement
                //Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y) + " Z" + DecimalPlaces(5,z) + " I" + DecimalPlaces(5,CX) + " J" + DecimalPlaces(5,CY));
                Memo1->Lines->Add("(Complete circles down to the bottom of the thread)");

                //while (z > (CZ - L)) {
                while (floor(z*10000) > floor((CZ - L)*10000)) {
                    oldx = x;
                    oldy = y;
                    oldz = z;
                    z -= P;
                    switch (CentreType) {
                        case TAbsolute: {
                                        Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y) + " Z" + DecimalPlaces(5,z) + " I" + DecimalPlaces(5,CX) + " J" + DecimalPlaces(5,CY));
                                        }
                                        break;
                        case TCP1:      {
                                        Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,CX - oldx) + " J" + DecimalPlaces(5,CY - oldy));
                                        }
                                        break;
                        case TP1C:      {
                                        Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,oldx - CX) + " J" + DecimalPlaces(5,oldy - CY));
                                        }
                                        break;
                        case TCP2:      {
                                        Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,CX - x) + " J" + DecimalPlaces(5,CY - y));
                                        }
                                        break;
                        case TP2C:      {
                                        Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,x - CX) + " J" + DecimalPlaces(5,y - CY));
                                        }
                                        break;
                        default:        {
                                        Memo1->Lines->Add("(Unknown Incremental Arc Definition Encountered. Please contact Technical Support at www.chestnutpens.co.uk)");
                                        }
                                        break;
                        }
                    //Memo1->Lines->Add("G2 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y) + " Z" + DecimalPlaces(5,z) + " I" + DecimalPlaces(5,CX) + " J" + DecimalPlaces(5,CY));
                    }

                //This start of thread now complete, feed radially inward to give a clearance of P
                x = CX;
                y = CY;
                Memo1->Lines->Add("(Feed away from end of Thread " + AnsiString(i + 1) + " to thread centre)");
                if (CentreType != TAbsolute) {
                    Memo1->Lines->Add("G90");
                    if (showG90point1) Memo1->Lines->Add("G90.1");
                    }
                Memo1->Lines->Add("G1 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y));

                //Rapid up to clearance height
                z = CZ + G + toolL1;
                Memo1->Lines->Add("(Rapid to Clearance Height)");
                Memo1->Lines->Add("G0 Z" + AnsiString(z));
                }
                //*/
            } else {
            //Revised version, all in degrees, cut starts at bottom of hole working to the top

            double total_number_of_rotations = (G + L + toolL1)/P;
            int total_number_of_full_rotations = (int)(total_number_of_rotations);
            double number_of_rotations_when_cutting = L/P;
            double start_angle = SA;
            double start_angle_of_cut = start_angle - 360.0 * number_of_rotations_when_cutting;
            double end_angle_of_helix = start_angle_of_cut + 360.0 * total_number_of_rotations;
            double z_at_start_of_cut = CZ - L;
            //double z_at_end_of_full_rotations = z_at_start_of_cut + ((double)(total_number_of_full_rotations)) * P;
            double z_at_end_of_helix = z_at_start_of_cut + total_number_of_rotations * P;
            anglebetweenstarts = 360.0 / (double)(N);
            z = z_at_end_of_helix;


            for (int i = 0; i < N; i++) {
                Memo1->Lines->Add("(Thread Start " + AnsiString(i+1) + ")");
                //already always should be at clearance height, but just in case
                if (z != z_at_end_of_helix) {
                    Memo1->Lines->Add("(Rapid to Clearance Height)");
                    Memo1->Lines->Add("G0 Z" + AnsiString(z_at_end_of_helix));
                    }
                Memo1->Lines->Add("(Rapid to Hole Centre)");
                Memo1->Lines->Add("G0 X" + DecimalPlaces(5,CX) + " Y" + DecimalPlaces(5,CY));
                Memo1->Lines->Add("(Feed to Bottom of Hole)");
                Memo1->Lines->Add("G1 Z" + DecimalPlaces(5,z_at_start_of_cut));
                x = CX + radius * cosD(start_angle_of_cut + ((double)(i)) * anglebetweenstarts);
                y = CY + radius * sinD(start_angle_of_cut+ ((double)(i)) * anglebetweenstarts);
                z = z_at_start_of_cut;
                oldx = x;
                oldy = y;
                oldz = z;
                Memo1->Lines->Add("(Feed radially to start of cut)");
                Memo1->Lines->Add("G1 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y));

                for (int j = 0; j < total_number_of_full_rotations; j++) {
                    z = z_at_start_of_cut + ((double) (j + 1)) * P;
                    switch (CentreType) {
                        case TAbsolute: {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y) + " Z" + DecimalPlaces(5,z) + " I" + DecimalPlaces(5,CX) + " J" + DecimalPlaces(5,CY));
                                        }
                                        break;
                        case TCP1:      {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,CX - oldx) + " J" + DecimalPlaces(5,CY - oldy));
                                        }
                                        break;
                        case TP1C:      {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,oldx - CX) + " J" + DecimalPlaces(5,oldy - CY));
                                        }
                                        break;
                        case TCP2:      {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,CX - x) + " J" + DecimalPlaces(5,CY - y));
                                        }
                                        break;
                        case TP2C:      {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,x - CX) + " J" + DecimalPlaces(5,y - CY));
                                        }
                                        break;
                        default:        {
                                        Memo1->Lines->Add("(Unknown Incremental Arc Definition Encountered. Please contact Technical Support at www.chestnutpens.co.uk)");
                                        }
                                        break;
                        }  //end 'switch (CentreType)'
                    oldx = x;
                    oldy = y;
                    oldz = z;
                    }    //end 'for (int j = 0; j < total_number_of_full_rotations; j++)'

                //now the last segment
                x = CX + radius * cosD(end_angle_of_helix + ((double)(i)) * anglebetweenstarts);
                y = CY + radius * sinD(end_angle_of_helix+ ((double)(i)) * anglebetweenstarts);
                z = z_at_end_of_helix;
                if (z != oldz) {
                    Memo1->Lines->Add("(Final arc to Clearance Height)");
                    switch (CentreType) {
                        case TAbsolute: {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x) + " Y" + DecimalPlaces(5,y) + " Z" + DecimalPlaces(5,z) + " I" + DecimalPlaces(5,CX) + " J" + DecimalPlaces(5,CY));
                                        }
                                        break;
                        case TCP1:      {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,CX - oldx) + " J" + DecimalPlaces(5,CY - oldy));
                                        }
                                        break;
                        case TP1C:      {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,oldx - CX) + " J" + DecimalPlaces(5,oldy - CY));
                                        }
                                        break;
                        case TCP2:      {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,CX - x) + " J" + DecimalPlaces(5,CY - y));
                                        }
                                        break;
                        case TP2C:      {
                                        Memo1->Lines->Add("G3 X" + DecimalPlaces(5,x - oldx) + " Y" + DecimalPlaces(5,y - oldy) + " Z" + DecimalPlaces(5,z - oldz) + " I" + DecimalPlaces(5,x - CX) + " J" + DecimalPlaces(5,y - CY));
                                        }
                                        break;
                        default:        {
                                        Memo1->Lines->Add("(Unknown Incremental Arc Definition Encountered. Please contact Technical Support at www.chestnutpens.co.uk)");
                                        }
                                        break;
                        }   //end 'switch'
                    } //end 'if (z != oldz)'

                } //end 'for (int i = 0; i < N; i++)'


            Memo1->Lines->Add("(Move to Thread Centre to Finish)");
            Memo1->Lines->Add("G0 X" + DecimalPlaces(5,CX) + " Y" + DecimalPlaces(5,CY));
            //end of new section
            } //end of 'if (ClimbMill) else...

        //Now to finish G Code

        if (GCodeFooter.Length()) {
            Memo1->Lines->Add("(Footer)");
            Memo1->Lines->Add(GCodeFooter);
            }
        } //end of 'if (P != 0)'

    //rewind memo to start.
    Memo1->SelStart = 0;
    Memo1->SelLength = 0;
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::ArcCentreDefinition1Click(TObject *Sender)
{
    Absolute1->Checked = (CentreType == (TCentreType) Absolute1->Tag);
    CP11->Checked = (CentreType == (TCentreType) CP11->Tag);
    P1C1->Checked = (CentreType == (TCentreType) P1C1->Tag);
    CP21->Checked = (CentreType == (TCentreType) CP21->Tag);
    P2C1->Checked = (CentreType == (TCentreType) P2C1->Tag);
    
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::Absolute1Click(TObject *Sender)
{
    //An arc centre definition type has been chosen
    CentreType = (TCentreType)  dynamic_cast<TMenuItem *>(Sender)->Tag;
    Recalculate(Sender);
    
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::ShowG9011Click(TObject *Sender)
{
    //don't want to show G90.1 & G91.1 codes (old machine controller)
    showG90point1 = !showG90point1;
    Recalculate(Sender);

}
//---------------------------------------------------------------------------

void __fastcall TIntForm::StatusBar1Hint(TObject *Sender)
{
    if (GetLongHint(Application->Hint).Length() != 0) {
        StatusBar1->Panels->Items[0]->Text = " " + GetLongHint(Application->Hint);
        } else {
        StatusBar1->Panels->Items[0]->Text = " ";
        }
    
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::Print1Click(TObject *Sender)
{
    if (PrintDialog1->Execute()) {
        TRichEdit *RichEdit1 = new TRichEdit(this);
        RichEdit1->Parent = IntForm;
        RichEdit1->Visible = false;
        RichEdit1->Font = Memo1->Font;
        RichEdit1->Lines->Text = Memo1->Lines->Text;
        RichEdit1->Print("Internal Thread G Code");
        delete RichEdit1;
        }
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::ThreadFit1Click(TObject *Sender)
{
    ExactSize1->Checked = (ThreadFit == (TFitType) ExactSize1->Tag);
    CloseFit1->Checked = (ThreadFit == (TFitType) CloseFit1->Tag);
    MediumFit1->Checked = (ThreadFit == (TFitType) MediumFit1->Tag);
    FreeFit1->Checked = (ThreadFit == (TFitType) FreeFit1->Tag);

}
//---------------------------------------------------------------------------

void __fastcall TIntForm::ExactSize1Click(TObject *Sender)
{
    ThreadFit = (TFitType) (dynamic_cast <TMenuItem *>(Sender)->Tag);
    Recalculate(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::ThreadDirection1Click(TObject *Sender)
{
    RightHand1->Checked = (ThreadDirection == (TThreadDir) RightHand1->Tag);
    LeftHand1->Checked = (ThreadDirection == (TThreadDir) LeftHand1->Tag);

}
//---------------------------------------------------------------------------

void __fastcall TIntForm::RightHand1Click(TObject *Sender)
{
    ThreadDirection = (TThreadDir) (dynamic_cast <TMenuItem *>(Sender)->Tag);
    Recalculate(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TIntForm::ClimbMill1Click(TObject *Sender)
{
    ClimbMill = !ClimbMill;
    Recalculate(Sender);
}
//---------------------------------------------------------------------------

